% This code solves the "encompassing model" in Chang and Fernandez (2010)
% On the Sources of Aggregate Fluctuations in Emerging Economies. NBER WP
% 15938. April, 2010. The code adapts the routines provided by Stephanie
% Schmitt-Grohe and Martin Uribe.

function [nfx,nfxp,nfy,nfyp] = encompassing(rhog,rhor,rhoz,mup,phi,eta,teta)

approx = 1; %Order of approximation desired

% %Define parameters
syms DELTA PHI MIUG RSTAR PSSI R RSTAR S TAU V SIGAMA BETTA B ALFA RHOZ S RHOR TETHA ETA RHOG SR

% %Define variables 
syms g  k  iv  b  w  h  u  q  c  la  k1  yy  z  nx  gy  gc  giv  dnx  yyl  cl  ivl  nxl  r  rstar  rl  s  gl  sr
syms gp kp ivp bp wp hp up qp cp lap k1p yyp zp nxp gyp gcp givp dnxp yylp clp ivlp nxlp rp rstarp rlp sp glp srp

%Write equations (e1, e2,...en)
e1  = -g*kp + (1-DELTA)*k + iv - (PHI/2)*k*(g*kp/k - MIUG)^2; 
e2  = -b + w*h + u*k + q*g*bp - iv - c;
e3  = -(q^-1) + r + PSSI*(exp(bp-B)-1);
e4  = -r + rstar*s;
e5  = -la + (c - TAU*h^V)^-SIGAMA;
e6  = -TAU*V*h^(V-1) + w;
e7  = -la*q + BETTA*(g^-SIGAMA)*lap;
e8  = -la*(1+PHI*(g*kp/k-MIUG)) + ...
    BETTA*(g^-SIGAMA)*lap*(up + 1 - DELTA + (PHI/2)*((gp*k1p/kp)^2 - MIUG^2));
e9  = -yy + z*k^(1-ALFA) * (g*h)^ALFA;
e10 = -u + (1-ALFA)*z*(k^-ALFA)*(g*h)^ALFA;
e11 = -w*(1 + TETHA*(rl-1)) + ALFA*z*(k^(1-ALFA))*(h^(ALFA-1))*(g^ALFA);
e12 = -log(nx) + (yy - iv - c)/yy; %Note: we take a linear, rather than log-linear, approximation with respect to nx
%growth and deviations observed in the measurement eq.
e13 = -gy + gl*yy/yyl;
e14 = -gc + gl*c/cl;
e15 = -giv + gl*iv/ivl;
e16 = -dnx + nx - nxl;
%Driving Forces
e17 = -log(zp) + RHOZ * log(z);
e18 = -log(gp/MIUG) + RHOG*log(g/MIUG); 
e19 = -log(rstarp/RSTAR) + RHOR*log(rstar/RSTAR); 
e20 = -sr + z*(g^ALFA);
e21 = -log(s/S) - ETA*log(srp/SR); 
%Aux. equations Linking Lags (1-period)
e22 = -k1 + kp;
e23 = yy - yylp; 
e24 = c  - clp; 
e25 = iv - ivlp; 
e26 = nx - nxlp; 
e27 = g - glp; 
e28 = r - rlp; 

% %Create function f
f = [e1;e2;e3;e4;e5;e6;e7;e8;e9;e10;e11;e12;e13;e14;e15;e16;e17;e18;e19;e20;e21;e22;e23;e24;e25;e26;e27;e28];
 
% Define the vector of controls, y, and states, x

xx  = [k  b  g  rstar  z  yyl  cl  ivl  nxl  rl  gl]; %11 States
xxp = [kp bp gp rstarp zp yylp clp ivlp nxlp rlp glp];
y   = [gy  gc  giv  dnx  r  q  c  h  iv  yy  nx  k1  la  w  u  s  sr]; %17 controls
yp  = [gyp gcp givp dnxp rp qp cp hp ivp yyp nxp k1p lap wp up sp srp];

%Make f a function of the logarithm of the state and control vector
f = subs(f, [xx,y,xxp,yp], exp([xx,y,xxp,yp]));

%Compute analytical derivatives of f
[fx,fxp,fy,fyp]=anal_deriv(f,xx,y,xxp,yp,approx);

%Numerical evaluation

%Assign values to parameters and steady-state variables
[ALFA,SIGAMA,BETTA,PHI,MIUG,DELTA,RSTAR,S,PSSI,B,RHOR,RHOZ,TAU,V,TETHA,ETA,RHOG,...
c,cp,h,hp,yy,yyp,q,qp,rstar,rstarp,k,kp,k1,k1p,b,bp,iv,ivp,z,zp,nx,nxp,la,lap,w,wp,u,up,r,rp,rl,rlp,s,sp,g,gp,gl,glp,...
gy,gyp,yyl,yylp,gc,gcp,cl,clp,giv,givp,ivl,ivlp,dnx,dnxp,nxl,nxlp,sr,srp,SR] = ...
encompassing_ss(rhog,rhor,rhoz,mup,phi,eta,teta);

%Compute numerical derivatives of f
num_eval